/***************************************************************************
                          mainwindow.cpp  -  description
                             -------------------
    begin                : Wed Mar 20 2002
    copyright            : (C) 2002 by Michael Herder
    email                : crapsite@gmx.net
 ***************************************************************************/

/*******************************************************************************
 *                                                                             *
 * This file is part of the example program "plugintest", which demonstrates   *
 * the usage of the QuiteInsane-Plugin.                                        *
 * It may be used, distributed and modified without limitation.                *
 *                                                                             *
 *******************************************************************************/

#include <qlabel.h>
#include <qlibrary.h>
#include <qmenubar.h>
#include <qpixmap.h>
#include <qpopupmenu.h>
#include <qptrlist.h>
#include <qsettings.h>

#include "mainwindow.h"

extern QLibrary* mpPluginLib;

MainWindow::MainWindow(QWidget* parent,const char* name)
           :QMainWindow(parent,name)
{
  QSettings settings;
  bool plugin_ok = false;
  mDeviceName = QString::null;

  if(!mpPluginLib)
    mpPluginLib = new QLibrary("quiteinsane_plugin_mt");
//if your using a Qt library without thread support, use the following code
//mpPluginLib = new QLibrary("quiteinsane_plugin");
  //try to resolve the init function
  quiteInsaneProc qisp = (quiteInsaneProc) mpPluginLib->resolve( "init_libquiteinsane_plugin" );
  //if the init function returns true, we are ready to use the plugin
  if(qisp)
  {
    qisp(this);
    plugin_ok = true;
    mDeviceName = settings.readEntry( "/Plugintest/device_name",QString::null);
  }
  else
   qDebug("could not resolve init function");
  mpLabel = new QLabel(this);
  setCentralWidget(mpLabel);
  QPopupMenu* pm=new QPopupMenu();
  int id;
  id = pm->insertItem(tr("&Setup scanner..."),this,
                      SLOT(slotSetupScanner()),0,1);
  //We gray-out the item, if the plugin isn't loaded
  if(!plugin_ok)
    pm->setItemEnabled(id,false);
  id = pm->insertItem(tr("Scan &image..."),this,
                      SLOT(slotScanImage()),0,2);
  //We gray-out the item, if the plugin isn't loaded
  if(!plugin_ok)
    pm->setItemEnabled(id,false);
  pm->insertSeparator();
  pm->insertItem(tr("&Quit"),this,SLOT(close()),
                 CTRL+Key_Q,3);
  menuBar()->insertItem(tr("&File"),pm);
}
MainWindow::~MainWindow()
{
}
void MainWindow::slotSetupScanner()
{
//To select a device, simply emit this signal
  emit signalQisGetDeviceName();
}
void MainWindow::slotScanImage()
{
//To start a scan, simply emit this signal
  emit signalQisScanImage(mDeviceName,true);
}
/**This slot is called, after the user is done with the device-selection dialog */
void MainWindow::slotQisNewDeviceName(QString device_name)
{
  mDeviceName = device_name;
  QSettings settings;
  settings.writeEntry( "/Plugintest/device_name",mDeviceName);
}
/**This slot is called, after the user is done with the scan-dialog */
void MainWindow::slotQisScanReady(QPtrList<QImage>& il)
{
  if(il.count() < 1)
    return;
  QImage* im = il.at(0);
  QPixmap pix;
  pix.convertFromImage(*im);
  mpLabel->setPixmap(pix);
  for(unsigned int i = 1;i < il.count();i++)
  {
    QLabel* l = new QLabel(this,0,Qt::WStyle_Customize|Qt::WType_TopLevel|Qt::WStyle_SysMenu|
                                  Qt::WStyle_MinMax|Qt::WStyle_Title);
    im = il.at(i);
    pix.convertFromImage(*im);
    l->setPixmap(pix);
    l->show();
  }
  il.clear();
}
