/***************************************************************************
                          quiteinsaneplugin.cpp  -  description
                             -------------------
    begin                : Fri Mar 22 2002
    copyright            : (C) 2002 by Michael Herder
    email                : crapsite@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "qdevicesettings.h"
#include "qscanner.h"
#include "qsanestatusmessage.h"
#include "qscandialog.h"
#include "qscannersetupdlg.h"
#include "quiteinsaneplugin.h"

#include <qapplication.h>
#include <qdir.h>
#include <qdom.h>
#include <qfileinfo.h>
#include <qwidget.h>

QString gSettingsKey = QString::null;
QString gSettingsPath = QString::null;

QuiteInsanePlugin::QuiteInsanePlugin(QWidget* parent)
                  :QObject(parent,parent->name())
{
  gSettingsKey = "/quiteinsane_plugin/";
  gSettingsKey += qApp->argv()[0];
  gSettingsKey += "/";
  mIsInit = false;
  mpScannerSetupDialog = 0;
  mpScanDialog = 0;
  if(initPlugin())
  {
    if(connect(parent,SIGNAL(signalQisScanImage(QString,bool)),
               this,SLOT(slotQisScanImage(QString,bool))) &&
       connect(this,SIGNAL(signalQisNewDeviceName(QString)),
               parent,SLOT(slotQisNewDeviceName(QString))) &&
       connect(parent,SIGNAL(signalQisGetDeviceName()),
               this,SLOT(slotQisGetDeviceName())) &&
       connect(this,SIGNAL(signalQisScanReady(QPtrList<QImage>&)),
               parent,SLOT(slotQisScanReady(QPtrList<QImage>&))))
    {
      mpScanner = new QScanner();
      mIsInit = true;
    }
    else
    {
      mpScanner = 0;
    }
  }
}
QuiteInsanePlugin::~QuiteInsanePlugin()
{
  if(mpScanDialog)
    delete mpScanDialog;
  if(mpScannerSetupDialog)
    delete mpScannerSetupDialog;
}
/** No descriptions */
void QuiteInsanePlugin::slotQisScanImage(QString device_name,bool allow_multiple_scans)
{
  QString dev;
  dev = device_name;
  if(!mpScanner)
    return;
  if(dev.isEmpty())
    dev = getDeviceName();
  if(dev.isEmpty())
    return;
  mDeviceName = dev;
  emit signalQisNewDeviceName(dev);
  mOptionMap.clear();
  if(mpScanner->name() != dev)
  {
    mpScanner->close();
    mpScanner->setDeviceName(dev);
    if(mpScanDialog)
    {
      delete mpScanDialog;
      mpScanDialog = 0;
    }
  }
  if(!mpScanner->isOpen())
  {
    if(!mpScanner->openDevice())
    {
      if(QScanner::msAuthorizationCancelled)
      {
        return;
      }
      QSaneStatusMessage status_msg(mpScanner->saneStatus(),(QWidget*)parent());
      status_msg.exec();
      return;
    }
  }
  if(!mpScanDialog)
  {
    loadLastSettings();
    if(!mOptionMap.isEmpty())
    {
       mpScanner->setOptionsByName(mOptionMap);
    }
    mpScanDialog = new QScanDialog(mpScanner,0);
  }
  if(mpScanDialog)
  {
    mpScanDialog->enableMultiSelection(allow_multiple_scans);
    mpScanDialog->exec();
    QDeviceSettings ds(mpScanner);
    ds.saveDeviceSettings("Last settings");
    QPtrList<QImage> il = mpScanDialog->imageList();
    emit signalQisScanReady(il);
  }
}
/** No descriptions */
void QuiteInsanePlugin::slotQisGetDeviceName()
{
  if(!mpScanner)
  {
    mDeviceName = QString::null;
    emit signalQisNewDeviceName(mDeviceName);
    return;
  }
  if(!mpScannerSetupDialog)
    mpScannerSetupDialog = new QScannerSetupDlg(mpScanner,(QWidget*)parent());
  if(mpScannerSetupDialog->exec())
  {
    mDeviceName = mpScannerSetupDialog->device();
    emit signalQisNewDeviceName(mDeviceName);
  }
  else
    emit signalQisNewDeviceName(QString::null);
}
/** No descriptions */
QString QuiteInsanePlugin::getDeviceName()
{
  if(!mpScanner)
    return QString::null;
  if(!mpScannerSetupDialog)
    mpScannerSetupDialog = new QScannerSetupDlg(mpScanner,(QWidget*)parent());
  if(mpScannerSetupDialog->exec())
  {
    return mpScannerSetupDialog->device();
  }
  return QString::null;
}
/** No descriptions */
bool QuiteInsanePlugin::initPlugin()
{
  QString qs;
  QString appname;
  char** appname_p;
  //We save our settings in a different directory, under a path
  //which is based on the applications name. This way, we avoid
  //conflicts between applications that use the plugin at the same time.
  appname_p = qApp->argv();
  appname = appname_p[0];
  QFileInfo fi(appname);
  appname = fi.fileName();
  qs = QDir::homeDirPath()+"/.quiteinsane_plugin/";
  QDir qd(qs);
  if(!qd.exists())
  {
    //try to create it
    if(!qd.mkdir(qs))
      return false;
  }
  qs.append(appname);
  qs.append("/");
  qd.setPath(qs);
  if(!qd.exists())
  {
    //try to create it
    if(!qd.mkdir(qs))
      return false;
  }
  gSettingsPath = qs;
  return true;
}

/** No descriptions */
void QuiteInsanePlugin::loadLastSettings()
{
//try to load the settings file
  QDomDocument doc;
  QString qs;
  doc.clear();
  QFile f( gSettingsPath+"devicesettings.xml" );
  if (f.open( IO_ReadOnly ) )
  {
    doc.setContent( &f );
    f.close();
  }
//The name of such a setting is unique for a given device.
  if(!doc.isNull())
  {//it's a setting
    //We search the specific entry in the xml file.
    QDomElement docElem = doc.documentElement();
    if(docElem.tagName() == "QuiteInsane_device_settings")
    {
      //Ok, now try to find the device settings
      QDomNode n = docElem.firstChild();
      while( !n.isNull() )
      {
        QDomElement e = n.toElement(); // try to convert the node to an element.
        if( !e.isNull() )
        { // the node was really an element.
          if(e.tagName() == "sane_device")
          {
            qs = e.attribute("username");
            //Search the device settings.
            if((mDeviceName == e.attribute("name"))   &&
               (qs == "Last settings"))
            {
              //We found  the entry.
              QDomNodeList dl = n.childNodes();
              for(unsigned int cnt=0;cnt<dl.count();cnt++)
              {
                QDomElement e2 = dl.item(cnt).toElement();
                if(e2.tagName() == "sane_option")
                {
                  mOptionMap.insert(e2.attribute("name"),
                                   e2.attribute("value"));
                }
              }
              break;
            }
          }
        }
        n = n.nextSibling();
      }
    }
  }
}
/** No descriptions */
bool QuiteInsanePlugin::isInit()
{
  return mIsInit;
}

extern "C"
{
  bool init_libquiteinsane_plugin(QWidget* obj)
  {
    QuiteInsanePlugin* d = new QuiteInsanePlugin(obj);
    if(!d->isInit())
    {
      delete d;
      return false;
    }
    return true;
  }
}
